from pyrediseasyio.io.single_io import SingleIO


class NumericIO(SingleIO):
    def __init__(self, name: str, addr: str = None, default: float = 0.0, **kwargs):
        """
        :param name:    A human readable name to give to the IO
        :param addr:    An optional address (key), if not given, them namespace + member name will be used
        :param default: The value to assign to the IO if no key is found during read
        :param kwargs:
         - units:       str: Optional units to assign to the IO
         - on_value:    str: The 'display_value' property will optionally return this value when the value is True
         - on_value:    str: The 'display_value' property will optionally return this value when the value is False
         - namespace:   str: Optional leading text to apply to the address to makes its key unique
         - min:              The smallest allowable number for the entry, reads and writes will be limited by this.
         - max:              The largest allowable number for the entry, reads and writes will be limited by this.
        """
        super().__init__(name, addr, default, **kwargs)

        self.min = kwargs.get('min')
        self.max = kwargs.get('max')


    def read(self):
        val = super().read()
        if val is None:
            return val
        if self.max is not None and val > self.max:
            val = self.max
        if self.min is not None and val < self.min:
            val = self.min
        return val

    def write(self, value):
        if value is None:
            return super().write(self, value)
        value = self._convert_type(value)
        if self.max is not None and value > self._convert_type(self.max):
            value = self.max
        if self.min is not None and value < self._convert_type(self.min):
            value = self.min
        super().write(value)




from pyrediseasyio.reader_writer import ReaderWriter
from pyrediseasyio.io.single_io import SingleIO
from typing import List, Callable
import json


class IOGroup(ReaderWriter):

    def __init__(self, host='localhost', port=6379, db=0,
                 set_defaults_on_startup: bool = False,
                 delete_keys_on_startup: bool = False,
                 namespace: str = None, *args, **kwargs):

        super().__init__(host=host, port=port, db=db, **kwargs)

        self.namespace = 'easyio' if namespace is None else namespace
        member_names = [d for d in dir(self) if not d.startswith('_')]
        self.members = []
        for name in member_names:
            try:
                attr = getattr(self, name)
                if not isinstance(attr, SingleIO):
                    continue
                attr._reader_writer = self
                self.members.append(name)
                if attr.addr is None:
                    attr.addr = name
                if namespace is not None:
                    attr.namespace = namespace
                if delete_keys_on_startup:
                    self.delete_key(attr.addr)
                if set_defaults_on_startup:
                    attr.write(attr.default)
            except AttributeError:
                pass

    def __len__(self):
        return len(self.members)

    def __setattr__(self, key, value):
        if hasattr(self, 'members') and key in self.members:
            attr = getattr(self, key)
            attr.write(value)
            return
        return super().__setattr__(key, value)

    def dump(self, key: str) -> str:
        """
        Creates a json string containing a list of dict(name/addr/value) of all members and stores it in-memory
        using the given key. The value can later be retrieved by a call to the 'read' method.
        :param key: A unique key for storing the dumped value
        :return: Returns the json string that is stored - for convenience sake.
        """
        s = self.dumps()
        self.write(key, s)
        return s

    def dumps(self, by_names: List = None, by_type: List = None, by_lambda: Callable = None) -> str:
        """
        Returns a json string containing a list of dict(name/addr/value) of all members.
        """
        attrs = self.get_attributes(by_names, by_type, by_lambda)
        members = [dict(name=attr.name, addr=attr.addr, value=attr.value) for attr in attrs]
        return json.dumps(members)

    def get_attribute(self, name: str=None, addr: str=None, key: str=None):
        if name:
            try:
                return getattr(self, name)
            except AttributeError:
                return None
        attr = None
        if addr:
            attr = self.get_attributes(by_lambda_each=lambda x: x.addr == addr)
        if key:
            attr = self.get_attributes(by_lambda_each=lambda x: x.key == key)
        if attr is not None and len(attr) > 0:
            return attr[0]
        return None

    def get_attributes(self, *args, **kwargs):
        """
        Gets a list of 'SingleIO' attributes for the group
        :param kwargs:
         - by_names: List[str]: Filters the list by attribute name
         - by_type:  List[SingleIO]: Filter the list by io type
         - by_lambda_each: Callable: Applies a filter to each attribute
         - by_lambda_results: Callable: Applies a filter to the resulting list (good for splicing)
        :return: List[SingleIO]
        """
        by_names = kwargs.get('by_names')
        by_type = kwargs.get('by_type')
        by_lambda_each = kwargs.get('by_lambda_each')
        by_lambda_results = kwargs.get('by_lambda_results')

        names = self.members if by_names is None else by_names
        attrs = [getattr(self, name) for name in names]

        if kwargs.get('by_type'):
            attrs = [a for a in attrs if type(a) in by_type]
        if kwargs.get('by_lambda_each'):
            attrs = [a for a in attrs if by_lambda_each(a)]
        if kwargs.get('by_lambda_results'):
            attrs = by_lambda_results(attrs)

        return attrs


    def set_all_to_defaults(self):
        """ Set the value for all members to their defaults."""
        attrs = self.get_attributes()
        for attr in attrs:
            attr.write(attr.default)









from pyrediseasyio.io.single_io import SingleIO
from typing import Callable


class TriggerIO(SingleIO):
    """ This IO does not actually write to the Redis server. Its intended use is to allow logic to be applied to its
    set or reset operations. For example, one might want to trigger a clearing of errors, in which case the set
    callback of the trigger would be wired to turning off several other IO"""

    def __init__(self, name: str, write_callback: Callable = None, read_callback: Callable = None,
                 error_callback: Callable = None, default = None, **kwargs):
        """
        :param name: A human readable name to be applied to the trigger
        :param write_callback: A method to call when writing to the trigger; of form - on_write(value) => return None
        :param read_callback: A method to call when reading from the trigger; of form - on_read() => return value
        :param error_callback: A method to call if there was an exception with in write/read callbacks;
                             of form - on_error(ex: Exception) => return substitute
        :param kwargs:
         - units:       str: Optional units to assign to the IO
         - on_value:    str: The 'display_value' property will optionally return this value when the value is True
         - on_value:    str: The 'display_value' property will optionally return this value when the value is False
        """
        super().__init__(name, addr=None, default=default, **kwargs)
        self.write_callback = write_callback
        self.read_callback = read_callback
        self.error_callback = error_callback
        self._iter_index = 0

    def read(self):
        if self.read_callback:
            try:
                return self.read_callback()
            except Exception as ex:
                if self.error_callback:
                    return self.error_callback(ex)
                else:
                    raise
        return self.default

    def write(self, value):
        if self.write_callback:
            try:
                return self.write_callback(value)
            except Exception as ex:
                if self.error_callback:
                    return self.error_callback(ex)
                else:
                    raise
        return None

    def __iter__(self):
        self._iter_index = 0
        return self

    def __next__(self):
        val = self.value
        if self._iter_index >= len(val):
            raise StopIteration
        else:
            self._iter_index += 1
            return val[self._iter_index - 1]




import unittest
from pyrediseasyio.io.io_group import IOGroup
from pyrediseasyio import BooleanIO, IntIO, FloatIO, StringIO, TriggerIO
from assertpy import assert_that
from htmldiffer import utils as diff_utils
import json


class TestGroup(IOGroup):
    Bool1 = BooleanIO("Boolean 1", "Bool1", False)
    Bool2 = BooleanIO("Boolean 2", "Bool2", True)
    Int1 = IntIO("Integer 1", "Int1")
    Int2 = IntIO("Integer 2", "Int2", default=34)
    Float1 = FloatIO("Float 1", "Float1", default=1.2)

    def __init__(self):
        super().__init__(channel="TestChannel")

    @staticmethod
    def on_reset_write(value):
        pass

    @staticmethod
    def on_reset_read():
        pass


class TestGroup2(IOGroup):
    Bool1 = BooleanIO("Boolean 1", "Bool1", False)
    Bool2 = BooleanIO("Boolean 2", "Bool2", True)
    Int1 = IntIO("Integer 1", "Int1")
    Int2 = IntIO("Integer 2", "Int2", default=34)
    Float1 = FloatIO("Float 1", "Float1", default=1.2)
    Message1 = StringIO("String 1", "String1")
    ApplyMany = TriggerIO("Apply to Many")


class TestTriggerIO(unittest.TestCase):

    def on_write(self, value):
        tf, i, f = value
        self.testgroup1.Bool1 = tf
        self.testgroup2.Bool1 = tf
        self.testgroup1.Int1 = i
        self.testgroup2.Int1 = i
        self.testgroup1.Float1 = f
        self.testgroup2.Float1 = f

    def on_read(self):
        tg1, tg2 = self.testgroup1, self.testgroup2
        return tg1.Int1, tg2.Float1

    def setUp(self) -> None:
        self.testgroup1 = TestGroup()
        self.testgroup2 = TestGroup2()
        self.testgroup2.ApplyMany.write_callback = self.on_write
        self.testgroup2.ApplyMany.read_callback = self.on_read

    def test_trigger_write(self):
        tg1, tg2 = self.testgroup1, self.testgroup2
        tg2.ApplyMany = (True, 100, 120.5)
        assert_that(tg1.Bool1).is_true()
        assert_that(tg2.Bool1).is_true()
        assert_that(tg1.Int1).is_equal_to(100)
        assert_that(tg2.Int1).is_equal_to(100)
        assert_that(tg1.Float1).is_equal_to(120.5)
        assert_that(tg2.Float1).is_equal_to(120.5)

    def test_trigger_read(self):
        tg1, tg2 = self.testgroup1, self.testgroup2
        tg1.Int1 = 333
        tg2.Float1 = 444.4
        a, b = tg2.ApplyMany
        assert_that(a).is_equal_to(333)
        assert_that(b).is_equal_to(444.4)





from abc import ABC, abstractmethod


class AbstractReaderWriter(ABC):

    @abstractmethod
    def publish(self, value: str, channel: str = None):
        """ Publishes a value to pub/sub """
        pass

    @abstractmethod
    def read(self, addr: str) -> object:
        """This method provides a response to a read request, based on last load"""
        pass

    @abstractmethod
    def write(self, addr: str, value: object):
        """This method captures the need to write a value, but isn't necessarily immediate"""
        pass








import dominate
import gettext
from dominate.util import raw

from pyrediseasyio.io.trigger_io import TriggerIO
from pyrediseasyio.io.single_io import SingleIO
from pyrediseasyio.io.boolean_io import BooleanIO
from pyrediseasyio.io.float_io import FloatIO
from dominate.tags import div, tr, td, button
from typing import List, Tuple

_ = gettext.gettext


class HTMLIO:


    def __init__(self, io: SingleIO):
        key = io.key
        self.io = io
        self.container_id = f'{key}_io'
        self.value_id = f'{key}'
        self.name = io.name
        self.value = io.value
        self.units = io.units
        self.namespace = io.namespace
        self.default_set_value = True if isinstance(io, BooleanIO) or isinstance(io, TriggerIO) else io.default
        self.default_reset_value = False if isinstance(io, BooleanIO) or isinstance(io, TriggerIO) else 0


    def _build(self, row_tag: dominate.tags, cell_tag: dominate.tags, **kwargs):
        show_name = bool(kwargs.get('show_name', True))
        show_value = bool(kwargs.get('show_value', True))
        show_set = bool(kwargs.get('show_set', False))
        show_reset = bool(kwargs.get('show_reset', False))
        show_change = bool(kwargs.get('show_change', False))
        show_units = bool(kwargs.get('show_units', False))
        set_text = str(kwargs.get('set_text', _("Set")))
        reset_text = str(kwargs.get('reset_text', _("Reset")))
        change_text = str(kwargs.get('change_text', _("Change")))

        io = self.io
        name, addr, val, units, ns = io.name, io.addr, io.value, io.units, self.namespace

        with row_tag(cls=f'easyio_io', id=self.container_id, data_type=type(io).__name__) as container:

            if show_name:
                self.name_cell(cell_tag)

            if show_value:
                self.value_cell(cell_tag)

            if show_units:
                units = '' if units is None else units
                cell_tag(units, cls=f'easyio_units')

            if show_set:
                with cell_tag(cls='easyio_btn_cell easyio_set_btn_cell'):
                    self.set_button(button, set_text, **kwargs)

            if show_reset:
                with cell_tag(cls='easyio_btn_cell easyio_rst_btn_cell'):
                    self.reset_button(button, reset_text, **kwargs)

            if show_change:
                with cell_tag(cls='easyio_btn_cell easyio_change_btn_cell'):
                    self.change_button(button, change_text, **kwargs)



        return container


    def as_divs(self, **kwargs) -> dominate.tags:
        """
        Creates html containing relevant parts of the IO using div elements
        :param kwargs:
            show_name:  bool: optional value, set to false to hide name column.
            show_value: bool: optional, set to false to hide the value column.
            show_set:   bool: optional, set to True to show the 'set' button column.
            show_reset: bool: optional, set to False to show the 'reset' button column.
            show_units: bool: optional, set to true so thow the units column.
            set_text:   str:  The text to place on the 'set' button
            reset_text: str:  The text to place on the 'reset' button)
            set_button_attributes: List[Tuple]: Allows additional attributes to be give to the set button
            reset_button_attributes: List[Tuple]: Allows additional attributes to be give to the reset button
            change_button_attributes: List[Tuple]: Allows additional attributes to be give to the change button
        :return: dominate tag object that can be rendered to html.
        """
        return self._build(div, div, **kwargs)


    def as_table_row(self, **kwargs) -> dominate.tags:
        """
        Creates html containing relevant parts of the IO using table elements
        :param kwargs:
            show_name:  bool: optional value, set to false to hide name column.
            show_value: bool: optional, set to false to hide the value column.
            show_set:   bool: optional, set to True to show the 'set' button column.
            show_reset: bool: optional, set to False to show the 'reset' button column.
            show_units: bool: optional, set to true so thow the units column.
            set_text:   str:  The text to place on the 'set' button
            reset_text: str:  The text to place on the 'reset' button)
            set_button_attributes: List[Tuple]: Allows additional attributes to be give to the set button
            reset_button_attributes: List[Tuple]: Allows additional attributes to be give to the reset button
            change_button_attributes: List[Tuple]: Allows additional attributes to be give to the change button
        :return: dominate tag object that can be rendered to html.
        """
        return self._build(tr, td, **kwargs)


    def change_button(self, tag: dominate.tags = button, txt: str = _("Change"),
                      change_button_attributes: List[Tuple] = None, **kwargs) -> dominate.tags:
        io = self.io
        name, addr, val, units, ns = io.name, io.addr, io.value, io.units, self.namespace
        ns = "na" if not ns else ns
        _max = io.max if hasattr(io, "max") else None
        _min = io.min if hasattr(io, "min") else None
        allow_decimal = isinstance(io, FloatIO)
        allow_negative = _min is None or _min < 0
        allow_negative = str(allow_negative).lower()
        allow_decimal = str(allow_decimal).lower()
        t = tag(txt, cls='easyio_change',
                   onclick=f"EasyIOChange('{io.name}', '{io.key}','{io.units}', '{_min}', "
                   f"'{_max}', {allow_decimal}, {allow_negative})")
        if change_button_attributes is not None:
            for a in change_button_attributes:
                t.attributes[a[0]] = a[1]
        return t


    def set_button(self, tag: dominate.tags = button, txt: str = "On", set_button_attributes: List[Tuple] = None,
                   **kwargs) -> dominate.tags:
        io = self.io
        name, addr, val, units, ns = io.name, io.addr, io.value, io.units, self.namespace
        ns = "na" if not ns else ns
        value = self.default_set_value
        t = tag(txt, cls='easyio_set', onclick=f"EasyIOSet('{ns}','{addr}','{self.value_id}','{value}')")
        if set_button_attributes is not None:
            for a in set_button_attributes:
                t.attributes[a[0]] = a[1]
        return t


    def reset_button(self, tag: dominate.tags = button, txt: str = "Off", reset_button_attributes: List[Tuple] = None,
                     **kwargs)-> dominate.tags:
        io = self.io
        name, addr, val, units, ns = io.name, io.addr, io.value, io.units, self.namespace
        ns = "na" if not ns else ns
        value = self.default_reset_value
        t = tag(txt, cls='easyio_reset', onclick=f"EasyIOSet('{ns}','{addr}','{self.value_id}','{value}')")
        if reset_button_attributes is not None:
            for a in reset_button_attributes:
                t.attributes[a[0]] = a[1]
        return t


    def with_custom_tags(self, outer: dominate.tags, inner: dominate.tags, **kwargs):
        """
        Creates html containing relevant parts of the IO using table elements
        :param outer:         The tag type to use for the cell container (row)
        :param inner:         The tag type to use for the cell
        :param kwargs:
            show_name:  bool: optional value, set to false to hide name column.
            show_value: bool: optional, set to false to hide the value column.
            show_set:   bool: optional, set to True to show the 'set' button column.
            show_reset: bool: optional, set to False to show the 'reset' button column.
            show_change: bool: optional, on numeric entries, adds a change button.
            show_units: bool: optional, set to true so thow the units column.
            set_text:   str:  The text to place on the 'set' button
            reset_text: str:  The text to place on the 'reset' button)
            set_button_attributes: List[Tuple]: Allows additional attributes to be give to the set button
            reset_button_attributes: List[Tuple]: Allows additional attributes to be give to the reset button
            change_button_attributes: List[Tuple]: Allows additional attributes to be give to the change button
        :return: dominate tag object that can be rendered to html.
        """
        return self._build(tr, td, **kwargs)


    def name_cell(self, tag: dominate.tags = div) -> dominate.tags:
        return tag(self.io.name, cls=f'easyio_name')


    def value_cell(self, tag: dominate.tags = div, allow_entry: bool=False) -> dominate.tags:
        """Generate the html for generating the cell that contains the object value."""
        io = self.io
        name, addr, val, units, ns = io.name, io.addr, io.value, io.units, self.namespace
        val = False if val is None else val
        t = tag(val, cls=f'easyio_value', id=self.value_id, onchange='OnEasyIOValueChange(event)',
                 data_namespace=ns, data_addr=addr)

        if io.namespace is not None:
            t['data-namespace'] = io.namespace
        if hasattr(io, 'max') and io.max is not None:
            t['data-max'] = io.max
        if hasattr(io, 'min') and io.min is not None:
            t['data-min'] = io.min
        if io.units is not None:
            t['data-units'] = io.units

        if allow_entry:
            t['onenter'] = 'OnEasyIOValueMouseEnter(event)'


from pyrediseasyio.abstract_reader_writer import AbstractReaderWriter
import threading
import json

lock = threading.Lock()


class SingleIO:
    def __init__(self, name: str, addr: str = None, default: object = None, **kwargs):
        """
        :param name:    A human readable name to give to the IO
        :param addr:    An optional address (key), if not given, them namespace + member name will be used
        :param default: The value to assign to the IO if no key is found during read
        :param kwargs:
         - units:       str: Optional units to assign to the IO
         - on_value:    str: The 'display_value' property will optionally return this value when the value is True
         - on_value:    str: The 'display_value' property will optionally return this value when the value is False
         - namespace:   str: Optional leading text to apply to the address to makes its key unique
        """
        self.name = name
        self.addr = addr
        self.namespace = kwargs.get('namespace')
        self._reader_writer = None
        self.default = default
        self.units = kwargs.get('units')

    def __and__(self, other):
        if hasattr(other, 'value'):
            return self.value and other.value
        return self.value and other

    def __add__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        return self.value + other

    def __sub__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        return self.value - other

    def __mul__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        return self.value * other

    def __eq__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        return self.value == other

    def __floordiv__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        return self.value // other

    def __iadd__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        val = self.value
        val += other
        self.write(val)

    def __idiv__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        val = self.value
        val /= other
        self.write(val)

    def __ifloordiv__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        val = self.value
        val //= other
        self.write(val)

    def __imul__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        val = self.value
        val *= other
        self.write(val)

    def __invert__(self):
        return ~self.value

    def __ipow__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        val = self.value
        val **= other
        self.write(val)

    def __isub__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        val = self.value
        val -= other
        self.write(val)

    def __ne__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        return self.value != other

    def __or__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        return self.value or other

    def __pos__(self):
        return self.value

    def __pow__(self, power, modulo=None):
        if hasattr(power, 'value'):
            power = power.value
        return self.value ** power

    def __neg__(self):
        return -self.value

    def __set__(self, obj, value):
        self.write(value)

    def __str__(self):
        units = '' if self.units is None else self.units
        return f'[{type(self).__name__}] {self.name} = {self.value} {units}'

    def __truediv__(self, other):
        if hasattr(other, 'value'):
            other = other.value
        return self.value / other

    @property
    def display_value(self):
        return self.value

    @property
    def key(self):
        if self.namespace is None:
            return self.addr
        return f'{self.namespace}{self.addr}'

    @property
    def value(self):
        return self.read()

    @staticmethod
    def _convert_type(value):
        return value

    def publish(self, value, channel: str = None, and_write: bool = False):
        value = self._convert_type(value)
        data = json.dumps({self.key: value})
        self._reader_writer.publish(data, channel)
        if and_write:
            self.write(value)

    def read(self):
        if self._reader_writer is None:
            return None
        with lock:
            val = self._reader_writer.read(self.key)
            if val is None:
                val = self.default
            val = self._convert_type(val)
            return val

    def write(self, value):
        if self._reader_writer is None:
            return
        with lock:
            value = self._convert_type(value)
            self._reader_writer.write(self.key, value)





from pyrediseasyio.io.single_io import SingleIO
from str2bool import str2bool
import gettext

_ = gettext.gettext


class BooleanIO(SingleIO):
    def __init__(self, name: str, addr: str = None, default: bool = False, **kwargs):
        """
        :param name:    A human readable name to give to the IO
        :param addr:    An optional address (key), if not given, them namespace + member name will be used
        :param default: The value to assign to the IO if no key is found during read
        :param kwargs:
         - units:       str: Optional units to assign to the IO
         - on_value:    str: The 'display_value' property will optionally return this value when the value is True
         - on_value:    str: The 'display_value' property will optionally return this value when the value is False
         - namespace:   str: Optional leading text to apply to the address to makes its key unique
        """
        super().__init__(name, addr, default, **kwargs)
        self.on_value = kwargs.get("on_value", _("On"))
        self.off_value = kwargs.get("off_value", _("Off"))

    @staticmethod
    def _convert_type(value):
        if value is None:
            return False
        if isinstance(value, str):
            return str2bool(value)
        return bool(value)

    def __bool__(self):
        return self.value


    @property
    def display_value(self):
        val = self.value
        dval = self.on_value if val else self.off_value
        return dval

    @property
    def value(self) -> bool:
        return super().value





if __name__ == "__main__":
    io = BooleanIO("Test")
    print(type(io))
    print(BooleanIO)
    print(type(io) == BooleanIO)


import unittest
from pyrediseasyio.io.io_group import IOGroup
from pyrediseasyio.html.html_io import HTMLIO
from pyrediseasyio import BooleanIO, SingleIO, FloatIO, StringIO
from assertpy import assert_that
from htmldiffer import utils as diff_utils
import json


class TestGroup(IOGroup):
    Bool1 = BooleanIO("Boolean 1", units="On/Off")
    Float1 = FloatIO("Float 1", default=1.23, units="furlongs")


test_group = TestGroup(namespace='Pin1')


def html_equals(actual, expected):
    lines1 = [line.strip() for line in diff_utils.html2list(actual)]
    lines1 = [line for line in lines1 if len(line) > 0]
    lines2 = [line.strip() for line in diff_utils.html2list(expected)]
    lines2 = [line for line in lines2 if len(line) > 0]
    diffs = [(line[0], line[1]) for line in zip(lines1, lines2) if line[0] != line[1]]
    return lines1 == lines2


class TestHTMLIO(unittest.TestCase):

    def test_can_add_set_button_attributes(self):
        attrs = [
            ('data-test', 'Test'),
            ('data-me', 'Me')
        ]
        h = HTMLIO(test_group.Bool1).as_divs(show_set = True, set_button_attributes=attrs)
        print(h)


from dominate import document
from dominate.tags import div, button
from collections import namedtuple
import gettext

_ = gettext.gettext

Btn = namedtuple("Btn", "cls val action")

keys = [
    Btn('number-button', _("1"), 1),
    Btn('number-button', _("2"), 2),
    Btn('number-button', _("3"), 3),
    Btn('action-button', _("Del"), 'Del'),
    Btn('number-button', _("4"), 4),
    Btn('number-button', _("5"), 5),
    Btn('number-button', _("6"), 6),
    Btn('action-button', _("Clear"), 'Clear'),
    Btn('number-button', _("7"), 7),
    Btn('number-button', _("8"), 8),
    Btn('number-button', _("9"), 9),
    Btn('action-button', _("Cancel"), 'Cancel'),
    Btn('number-button plus-minus-button', _("+/-"), 'Sign'),
    Btn('number-button', _("0"), 0),
    Btn('number-button decimal_button', _("."), 'Dec'),
    Btn('action-button ok-button', _("Ok"), 'Ok')
]

def keyboard(doc: document):
    with doc.body:
        with div(cls="easyio_number_keyboard"):
            with div(cls='grid-container'):
                div("Title", cls = "title")
                div(0, cls='entry', id="easy_io_number_keyboard_entry")
                div('', cls='original')

                for key in keys:
                    button(key.val, cls=key.cls, onclick=f"EasyIoNumericEntryKeyPressed('{key.action}')")

                div('', cls='max')
                div('', cls='min')
